/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 * Henrik Gemal http://gemal.dk/
 *
 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var LaunchyApps = function() {}

LaunchyApps.prototype.init = function() {
	LaunchyApps.progdir = Launchy_DirIO.get("CurProcD").path.toLowerCase();
	// add directory separator but only if we dont end in a separator
	if (!/(\\|\/)$/.test(LaunchyApps.progdir)) {
		LaunchyApps.progdir += Launchy_DirIO.sep;
	}
	launchyShared.trace("Program directory is <" + LaunchyApps.progdir + ">");

	if ("@mozilla.org/windows-registry-key;1" in Components.classes) {
		launchyShared.trace("Try using the windows-registry-key service!");
		try {
			LaunchyContext.winhooks = Components.classes["@mozilla.org/windows-registry-key;1"].getService(Components.interfaces.nsIWindowsRegKey);
			LaunchyContext.winhooksAPI = 2;
		} catch(ex) {
			launchyShared.trace("Unable to get windows-registry-key service!");
		}
	} else if ("@mozilla.org/winhooks;1" in Components.classes) {
		launchyShared.trace("Try using the winhooks service!");
		try {
			LaunchyContext.winhooks = Components.classes["@mozilla.org/winhooks;1"].getService(Components.interfaces.nsIWindowsRegistry);
			LaunchyContext.winhooksAPI = 1;
		} catch(ex) {
			launchyShared.trace("Unable to get winhooks service!");
		}
	} else if ("@mozilla.org/browser/shell-service;1" in Components.classes) {
		launchyShared.trace("Try using the shell-service service!");
		try {
			LaunchyContext.winhooks = Components.classes["@mozilla.org/browser/shell-service;1"].getService(Components.interfaces.nsIWindowsShellService);
			LaunchyContext.winhooksAPI = 1;
		} catch(ex) {
			launchyShared.trace("Unable to get browser shell service!");
		}
	} else {
		launchyShared.trace("Unable to get Components. This only works on Windows!");
	}
	if (typeof(LaunchyContext.winhooks) == "undefined" || (LaunchyContext.winhooksAPI == 1 && typeof(LaunchyContext.winhooks.getRegistryEntry) != "function") || (LaunchyContext.winhooksAPI == 2 && typeof(LaunchyContext.winhooks.open) != "function")) {
		LaunchyContext.winhooks = false;
		launchyShared.trace("Registry functions are't available. These only works on Windows!");
	} else {
		launchyShared.trace("Using winhooksAPI version <" + LaunchyContext.winhooksAPI + ">");
	}

	LaunchyContext.apps = new Array();
	LaunchyContext.appsnot = new Array();
	LaunchyContext.appsdisabled = new Array();
	LaunchyContext.appstype = new Array();
	LaunchyContext.appstype[0] = "";
	LaunchyContext.appstype[1] = launchyContext.getBundle("launchy-apps-label-browser", "bundle_launchy_apps");
	LaunchyContext.appstype[2] = launchyContext.getBundle("launchy-apps-label-mail", "bundle_launchy_apps");
	LaunchyContext.appstype[3] = launchyContext.getBundle("launchy-apps-label-media", "bundle_launchy_apps");
	LaunchyContext.appstype[4] = launchyContext.getBundle("launchy-apps-label-ftp", "bundle_launchy_apps");
	LaunchyContext.appstype[5] = launchyContext.getBundle("launchy-apps-label-download", "bundle_launchy_apps");
	LaunchyContext.appstype[6] = launchyContext.getBundle("launchy-apps-label-filemgr", "bundle_launchy_apps");
	LaunchyContext.appstype[7] = launchyContext.getBundle("launchy-apps-label-editor", "bundle_launchy_apps");
	LaunchyContext.appstype[8] = launchyContext.getBundle("launchy-apps-label-image", "bundle_launchy_apps");

	try {
		var disabled = LaunchyContext.pref.getCharPref("extensions.launchy.disabledapps");
		if (disabled) {
			launchyShared.trace("Reading disabledapps pref: " + disabled);
			LaunchyContext.disabledApps = disabled.split(",");
		}
	}	catch(ex) {
		launchyShared.trace("No disabledapps pref");
	}

	if (LaunchyContext.winhooks) {
		launchyShared.trace("Loading applications from JavaScript");

		// browers
		launchyApps.addApp(2, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla", "", 1, "", 1);
		launchyApps.addApp(2, "Software\\mozilla.org\\Mozilla Firebird", "CurrentVersion", "Mozilla Firebird", "", 1, "", 1);
		launchyApps.addApp(2, "Software\\Mozilla\\Mozilla Firefox", "CurrentVersion", "Mozilla Firefox", "", 1, "", 1);
		launchyApps.addApp(2, "Software\\Netscape\\Netscape", "CurrentVersion", "Netscape", "", 1, "", 1);
		launchyApps.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\IEXPLORE.EXE", "", "Internet Explorer", "", 1, "", 0);
		launchyApps.addApp(2, "Software\\Opera Software", "Last Directory3", "Opera", "opera.exe", 1, "", 0);
		launchyApps.addApp(2, "Software\\Opera Software", "Last Beta Directory", "Opera", "opera.exe", 1, "", 0);
		launchyApps.addApp(2, "Software\\K-Meleon\\K-Meleon\\General", "InstallDir", "K-Meleon", "k-meleon.exe", 1, "", 0);
		launchyApps.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Maxthon", "DisplayIcon", "Maxthon", "", 1, "", 0);
		launchyApps.addApp(2, "Software\\Siber Systems\\RoboForm", "_TBP_", "Avant Browser", "", 1, "", 0);
		launchyApps.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Crazy Browser_is1", "DisplayIcon", "Crazy Browser", "", 1, "", 0);
		launchyApps.addApp(3, "Software\\FlashPeak\\SlimBrowser", "", "Slim Browser", "sbrowser.exe", 1, "", 0);

		// mail clients
		launchyApps.addApp(2, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla Mail", "", 2, ["-compose"], 1);
		launchyApps.addApp(2, "Software\\Mozilla\\Mozilla Thunderbird", "CurrentVersion", "Mozilla Thunderbird", "", 2, ["-compose"], 1);
		launchyApps.addApp(3, "Software\\Mozilla\\Mozilla Thunderbird", "CurrentVersion", "Mozilla Thunderbird", "", 2, ["-compose"], 1);
		launchyApps.addApp(2, "Software\\Netscape\\Netscape", "CurrentVersion", "Netscape Mail", "", 2, ["-compose"], 1);
		launchyApps.addApp(2, "Software\\Pegasus Mail\\BaseDir", "", "Pegasus Mail", "wsendto.exe", 2, "", 0);
		launchyApps.addApp(3, "Software\\Lotus\\Notes", "Path", "Lotus Notes", "notes.exe", 2, "", 0);
		launchyApps.addApp(2, "Software\\Qualcomm\\Eudora\\CommandLine", "current", "Eudora", "", 2, "", 0);
		launchyApps.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\OUTLOOK.EXE", "", "Outlook", "", 2, ["-c", "IPM.Note", "/m"], 0);
		launchyApps.addApp(2, "Software\\RIT\\The Bat!", "EXE path", "The Bat!", "", 2, "", 0);
		launchyApps.addApp(3, "Software\\IncrediMail", "InstallationPath", "IncrediMail", "bin\\IncMail.exe", 2, ["/c", "/n", "/mailto:#1"] , 0);
		launchyApps.addApp(3, "Software\\MCS\\Calypso3", "Path", "Calypso", "Calypso.exe", 2, ["/MAILTO"] , 0);
		launchyApps.addApp(3, "Software\\Microsoft\\WAB\\Export\\CSV", "DLL", "Outlook Express", "", 2, ["/mailurl:#1"] , 0, "msimn.exe");
		launchyApps.addApp(2, "Software\\Poco Systems Inc\\PocoMail 3", "Path", "PocoMail", "poco.exe", 2, ["/m"] , 0);
		launchyApps.addApp(2, "Software\\Forte\\Agent\\Paths", "ExeFile", "Forte Agent", "", 2, ["-home:MailIniFile", "-url:#1"] , 0);

		// media players
		launchyApps.addApp(2, "Software\\BST\\bsplayerv1", "AppPath", "BSPlayer", "", 3, "", 0);
		launchyApps.addApp(3, "Software\\Clients\\Media\\iTunes\\shell\\open\\command", "", "iTunes", "", 3, ["/url"], 0);
		launchyApps.addApp(2, "Software\\Winamp", "", "Winamp", "winamp.exe", 3, "", 0);
		launchyApps.addApp(3, "Software\\Microsoft\\MediaPlayer", "Installation Directory", "Windows Media Player", "wmplayer.exe", 3, "", 0);
		launchyApps.addApp(3, "Software\\Gabest\\Media Player Classic", "ExePath", "Media Player Classic", "", 3, "", 0);
		launchyApps.addApp(0, "Applications\\wordview.exe\\shell\\Open\\command", "", "Word Viewer", "", 3, "", 0);
		launchyApps.addApp(0, "Applications\\xlview.exe\\shell\\Open\\command", "", "Excel Viewer", "", 3, "", 0);
		launchyApps.addApp(2, "Software\\foobar2000", "InstallDir", "Foobar2000", "foobar2000.exe", 3, "", 0);

		// FTP clients
		launchyApps.addApp(2, "Software\\Ghisler\\Total Commander", "InstallDir", "Total Commander", "totalcmd.exe", 4, "", 0);
		launchyApps.addApp(2, "Software\\UltraFXP", "", "UltraFXP", "ultrafxp.exe", 4, "", 0);
		launchyApps.addApp(3, "Software\\Ipswitch\\WS_FTP", "dir", "WS_FTP Pro", "wsftpgui.exe", 4, "", 0);
		launchyApps.addApp(3, "Software\\SmartFTP", "Install Directory", "SmartFTP", "smartftp.exe", 4, "", 0);
		launchyApps.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\BulletProof FTP_is1", "DisplayIcon", "BulletProof FTP", "", 4, "", 0);
		launchyApps.addApp(2, "Software\\FileZilla", "Install_Dir", "FileZilla", "FileZilla.exe", 4, "", 0);
		launchyApps.addApp(2, "Software\\VanDyke\\AbsoluteFTP\\Install", "Main Directory", "AbsoluteFTP", "AbsoluteFTP.EXE", 4, "", 0);
		launchyApps.addApp(2, "Software\\Altap\\Applications\\Servant Salamander 2.5 beta 7", "LastDirectory", "Servant Salamander", "salamand.exe", 4, ["-R"], 0);
		launchyApps.addApp(2, "Software\\FlashFXP\\3", "Install Path", "FlashFXP", "flashfxp.exe", 4, "", 0);

		// download managers
		launchyApps.addApp(2, "Software\\HeadLight\\GetRight\\Config", "InstallDir", "GetRight", "getright.exe", 5, "", 0);
		launchyApps.addApp(2, "Software\\Cronosoft\\LeechGet\\Console", "LeechGetPath", "LeechGet", "", 5, ["-wizard"], 0);
		launchyApps.addApp(0, "TypeLib\\{3F9A5301-9E5A-11D3-A288-0000E80E2EDE}\\1.0\\HELPDIR", "", "Mass Downloader", "massdown.exe", 5, "", 0);
		launchyApps.addApp(2, "Software\\Star Downloader", "Path", "Star Downloader", "", 5, "", 0);
		launchyApps.addApp(2, "Software\\DownloadManager", "ExePath", "Internet Download Manager", "", 5, ["/d"], 0);
		launchyApps.addApp(0, "TypeLib\\{1537E849-0200-11D2-8059-000000000000}\\1.0\\HELPDIR", "", "ReGet Deluxe", "regetdx.exe", 5, "", 0);
		launchyApps.addApp(0, "bittorrent\\shell\\open\\command", "", "BitTorrent", "", 5, ["--url"], 0);
		launchyApps.addApp(2, "Software\\JetCar\\JetCar\\General", "AppPath", "FlashGet", "", 5, "", 0);
		launchyApps.addApp(2, "Software\\Radiate\\Go!Zilla", "", "Go!Zilla", "", 5, "", 0);
		launchyApps.addApp(3, "Software\\SpeedBit\\Download Accelerator", "EXELOCATION", "Download Accelerator", "", 5, "", 0);
		launchyApps.addApp(3, "Software\\Millweed\\WackGet", "", "WackGet", "WackGet.exe", 5, "", 0);
		launchyApps.addApp(2, "Software\\MetaProducts\\Offline Explorer Pro\\Parameters", "LastRunPos", "Offline Explorer Pro", "", 5, ["/new=url=#1"], 0);
		launchyApps.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\NetTransport_is1\\Inno Setup", "App Path", "NetTransport", "", 5, "", 0);

		// file explorers
		launchyApps.addApp(2, "Software\\Ghisler\\Total Commander", "InstallDir", "Total Commander", "totalcmd.exe", 6, "", 0);
		launchyApps.addApp(3, "Software\\Microsoft\\Windows NT\\CurrentVersion", "SystemRoot", "Explorer", "explorer.exe", 6, "", 0);
		launchyApps.addApp(2, "Software\\Altap\\Applications\\Servant Salamander 2.5 beta 7", "LastDirectory", "Servant Salamander", "salamand.exe", 6, "", 0);

		// editors
		launchyApps.addApp(0, "Applications\\notepad.exe\\shell\\edit\\command", "", "Notepad", "", 7, "", 0);
		launchyApps.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\UEDIT32.exe", "", "UltraEdit", "", 7, "", 0);
		launchyApps.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\soffice.exe", "", "OpenOffice.org", "", 7, ["-o"], 0);
		launchyApps.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\soffice.exe", "", "OpenOffice.org", "", 7, ["-o"], 0);
		launchyApps.addApp(2, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla Composer", "", 7, ["-editor"], 1);
		launchyApps.addApp(2, "Software\\Netscape\\Netscape", "CurrentVersion", "Netscape Composer", "", 7, ["-editor"], 1);
		launchyApps.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\WORDPAD.EXE", "", "WordPad", "", 7, "", 0);
		launchyApps.addApp(2, "Software\\Ascher\\syn", "", "Syn Text Editor", "", 7, "", 0);
		launchyApps.addApp(0, "TextPad.tws\\DefaultIcon", "", "TextPad", "", 7, "", 0);
		launchyApps.addApp(3, "Software\\Zend Technologies\\Studio", "ZDEPath", "Zend Studio", "", 7, "", 0);
		launchyApps.addApp(0, "CLSID\\{0F0BFAE1-4C90-11D1-82D7-0004AC368519}\\LocalServer32", "", "Vim", "", 7, "", 0);
		launchyApps.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\PSPad editor_is1", "InstallLocation", "PSPad", "pspad.exe", 7, "", 0);
		launchyApps.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Nvu_is1", "InstallLocation", "NVU", "nvu.exe", 7, ["-edit"], 0);
		launchyApps.addApp(2, "Software\\ES-Computing\\EditPlus 2\\Install", "Path", "EditPlus", "editplus.exe", 7, "", 0);

		// image viewers
		launchyApps.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\XnView_is1", "Inno Setup: App Path", "XnView", "xnview.exe", 8, "", 0);
		launchyApps.addApp(2, "Software\\Jasc\\Global", "PSP8DefaultRead", "Paint Shop Pro", "Paint Shop Pro.exe", 8, "", 0);
		launchyApps.addApp(2, "Software\\Jasc\\Global", "PSP9DefaultRead", "Paint Shop Pro", "Paint Shop Pro 9.exe", 8, "", 0);
		launchyApps.addApp(0, "Applications\\i_view32.exe\\shell\\open\\command", "", "IrfanView", "", 8, "", 0);
	}
	// load apps from XML file
	launchyApps.loadApps();

}

LaunchyApps.prototype.getRegEntry = function(key, path, name) {
	var reg = new Array(0x80000000, 0x80000005, 0x80000001, 0x80000002, 0x80000003);
	// reg values:
	// 0: HKEY_CLASSES_ROOT = 0x80000000
	// 1: HKEY_CURRENT_CONFIG = 0x80000005
	// 2: HKEY_CURRENT_USER = 0x80000001
	// 3: HKEY_LOCAL_MACHINE = 0x80000002
	// 4: HKEY_USERS = 0x80000003

	var tmp;
	if (LaunchyContext.winhooksAPI == 1) {
		return LaunchyContext.winhooks.getRegistryEntry(key, path, name);
	} else if (LaunchyContext.winhooksAPI == 2) {
		var regkey = Components.classes["@mozilla.org/windows-registry-key;1"].createInstance(Components.interfaces.nsIWindowsRegKey);
		try {
			regkey.open(reg[key], path, Components.interfaces.nsIWindowsRegKey.ACCESS_READ);
			if (regkey.valueCount) {
				try {
					return regkey.readStringValue(name);
				} catch(ex) {
				}
			}
		} catch(ex) {
		}
	} else {
		launchyShared.trace("Unknown winhooksAPI version");
	}
	return false;
}

LaunchyApps.prototype.substWinPaths = function(path) {
	if (LaunchyContext.winhooks) {
		if (/%ProgramFiles%/i.test(path)) {
			var ProgramFiles = launchyApps.getRegEntry(3, "Software\\Microsoft\\Windows\\CurrentVersion", "ProgramFilesDir");
			var re=/%ProgramFiles%/i;
			path = path.replace(re, ProgramFiles);
		}
		if (/%SystemRoot%/i.test(path)) {
			var SystemRoot = launchyApps.getRegEntry(3, "Software\\Microsoft\\Windows NT\\CurrentVersion", "SystemRoot");
			re=/%SystemRoot%/i;
			path = path.replace(re, SystemRoot);
		}
		if (/%HOMEDRIVE%/i.test(path)) {
			var HOMEDRIVE = launchyApps.getRegEntry(2, "Volatile Environment", "HOMEDRIVE");
			re=/%HOMEDRIVE%/i;
			path = path.replace(re, HOMEDRIVE);
		}
		if (/%HOMEPATH%/i.test(path)) {
			var HOMEPATH = launchyApps.getRegEntry(2, "Volatile Environment", "HOMEPATH");
			re=/%HOMEPATH%/i;
			path = path.replace(re, HOMEPATH);
		}
		if (/%GeckoDrive%/i.test(path)) {
			var GeckoDrive = LaunchyApps.progdir.substring(0, 2);
			re=/%GeckoDrive%/i;
			path = path.replace(re, GeckoDrive);
		}
	}
	return path;
}

LaunchyApps.prototype.findApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_gecko, filereplace) {
	var reg_type = new Array("HKEY_CLASSES_ROOT", "HKEY_CURRENT_CONFIG", "HKEY_CURRENT_USER", "HKEY_LOCAL_MACHINE", "HKEY_USERS");
	launchyShared.trace("Looking for <" + app_name + "> in <" + reg_type[reg] + "> at <" + reg_sub + (reg_name?"\\"+reg_name:"") + ">" + (parseInt(app_gecko)?" and it's a Gecko":""));
	try {
		var tmp = launchyApps.getRegEntry(reg, reg_sub, reg_name);
		var tmp_gecko;
		if (app_gecko && tmp) {
			launchyShared.trace("Found Gecko version <" + tmp + ">");
			tmp_gecko = tmp;
			tmp = launchyApps.getRegEntry(reg, reg_sub + "\\" + tmp + "\\Main", "PathToExe");
		}
	} catch(ex) {
		launchyShared.trace("Unable to find <" + app_name + ">");
		return false;
	}
	if (tmp) {
		tmp = launchyApps.substWinPaths(tmp);

		var re=/\\\\/;
		tmp = tmp.replace(re, "\\");
		re=/\\$/;
		tmp = tmp.replace(re, "");
		re=/%1$/;
		tmp = tmp.replace(re, "");
		re=/\s+$/;
		tmp = tmp.replace(re, "");

		launchyShared.trace("<" + app_name + "> registry entry found <" + tmp + ">");
		if (app_exe) {
			tmp += "\\" + app_exe;
		}

		var x;
		// if value contains a comma like: c:\temp\test.exe, 2
		re=/(.*?)?,/;
		x = tmp.match(re);
		if (x && x[1]) {
			launchyShared.trace("Comma removed");
			tmp = x[1];
		}

		// if value is inside quotes like: "c:\temp\test.exe"
		re=/\"(.*?)\" ?/;
		x = tmp.match(re);
		if (x && x[1]) {
			launchyShared.trace("Quotes removed");
			tmp = x[1];
		}

		// if value contains parameters like: c:\temp\test.exe /c
		re=/(.*?) \/.*?$/;
		x = tmp.match(re);
		if (x && x[1]) {
			launchyShared.trace("Parameters removed");
			tmp = x[1];
		}

		// if value contains path paramters like: c:\temp\test.exe c:\temp
		re=/(.*?) [a-zA-Z]:\\/;
		x = tmp.match(re);
		if (x && x[1]) {
			launchyShared.trace("Path paramters removed");
			tmp = x[1];
		}

		if (filereplace) {
			re=/\w+\.\w+$/;
			tmp = tmp.replace(re, filereplace);
		}

		var ft = Launchy_FileIO.open(tmp);
		if (ft && ft.exists()) {
			if (ft.isFile() && ft.isExecutable()) {
				launchyShared.trace(app_name + (app_gecko ? " <" + tmp_gecko + ">" : "") + " is installed at <" + tmp + ">");
				if (app_gecko && tmp.toLowerCase().indexOf(LaunchyApps.progdir) != -1) {
					launchyShared.trace("Hide " + app_name + " since it's the running client");
					return false;
				}
			} else {
				launchyShared.trace("File <" + tmp + "> isn't a file or isn't executable");
				return false;
			}
			return tmp;
		} else {
			launchyShared.trace(app_name + " isn't at: <" + tmp + ">");
			return false;
		}
	} else {
		launchyShared.trace(app_name + " isn't installed or undetected");
		return false;
	}
}

LaunchyApps.prototype.createApp = function(app_name, app_exe, app_type, app_args, app_icon) {
	var app = new Array();
	app.name = app_name;
	app.exe = app_exe;
	app.type = app_type;
	app.args = app_args;
	app.icon = app_icon;
	return app;
}

LaunchyApps.prototype.addApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_type, app_args, app_gecko, filereplace) {
	// Check for dupes
	for (var i = 0; LaunchyContext.apps.length > i; i++) {
		if (LaunchyContext.apps[i].name == app_name) {
			if (LaunchyContext.apps[i].type != app_type) {
				launchyShared.trace("Reusing <" + app_name + "> detection");
				LaunchyContext.apps[LaunchyContext.apps.length] = launchyApps.createApp(app_name, LaunchyContext.apps[i].exe, app_type, app_args);
				return;
			} else {
				launchyShared.trace("<" + app_name + "> type <" + app_type + "> is already detected");
				return;
			}
		}
	}

	var tmp = launchyApps.findApp(reg, reg_sub, reg_name, app_name, app_exe, app_gecko, filereplace);
	// Check for disabled apps
	if (tmp) {
		if (typeof(LaunchyContext.disabledApps) != "undefined") {
			for (i = 0; i < LaunchyContext.disabledApps.length; ++i) {
				if (LaunchyContext.disabledApps[i] == app_name) {
					LaunchyContext.appsdisabled[LaunchyContext.appsdisabled.length] = launchyApps.createApp(app_name, tmp, app_type, app_args);
					return;
				}
			}
		}
		LaunchyContext.apps[LaunchyContext.apps.length] = launchyApps.createApp(app_name, tmp, app_type, app_args);
	} else {
		LaunchyContext.appsnot[LaunchyContext.appsnot.length] = launchyApps.createApp(app_name, "", app_type, app_args);
	}
}

LaunchyApps.prototype.loadApps = function() {
	var xmlfile;
	try {
		xmlfile = LaunchyContext.pref.getCharPref("extensions.launchy.xmlfile");
	} catch(ex) {
	}
	if (typeof(xmlfile) != "undefined" && xmlfile) {
		launchyShared.trace("Launchy XML file specified");
	} else {
		var chromedir = Launchy_DirIO.get("UChrm");
		chromedir.append("launchy.xml");
		xmlfile = chromedir.path;
	}
	launchyShared.trace("Looking for <" + xmlfile + ">");
	var f = Launchy_FileIO.open(xmlfile);
	if (f && f.exists()) {
		launchyShared.trace("Loading applications from <" + xmlfile + ">");
		var con = Launchy_FileIO.read(f);
		if (!con) {
			launchyShared.trace("Unable to read file");
			return;
		}
		var domParser=Components.classes["@mozilla.org/xmlextras/domparser;1"].createInstance(Components.interfaces.nsIDOMParser);
		var apps = domParser.parseFromString(con, "text/xml");
		if (apps.firstChild.nodeName == "parsererror") {
			launchyShared.trace("Invalid launchy.xml file:\n"+ apps.firstChild.textContent, 1);
			return;
		}
		var c = apps.getElementsByTagName("configurations");
		if (c) {
			var a = apps.getElementsByTagName("application");
			if (a) {
				for (var i = 0; i < a.length; i++) {
					var tmp;
					var app_name = "";
					tmp = a.item(i).getElementsByTagName("label");
					if (tmp.item(0) && tmp.item(0).firstChild && tmp.item(0).firstChild.nodeValue)
						app_name = tmp.item(0).firstChild.nodeValue;

					var app_type = "";
					tmp = a.item(i).getElementsByTagName("type");
					if (tmp.item(0) && tmp.item(0).firstChild && tmp.item(0).firstChild.nodeValue)
						app_type = tmp.item(0).firstChild.nodeValue;

					var app_exe = "";
					tmp = a.item(i).getElementsByTagName("command");
					if (tmp.item(0) && tmp.item(0).firstChild && tmp.item(0).firstChild.nodeValue) {
						app_exe = tmp.item(0).firstChild.nodeValue;
						if (app_exe)
							app_exe = launchyApps.substWinPaths(app_exe);
					}

					var app_args = "";
					tmp = a.item(i).getElementsByTagName("arguments");
					if (tmp.item(0) && tmp.item(0).firstChild && tmp.item(0).firstChild.nodeValue) {
						app_args = tmp.item(0).firstChild.nodeValue;
						if (app_args)
							app_args = launchyApps.substWinPaths(app_args);
					}

					var app_icon = "";
					tmp = a.item(i).getElementsByTagName("icon");
					if (tmp.item(0) && tmp.item(0).firstChild && tmp.item(0).firstChild.nodeValue) {
						app_icon = tmp.item(0).firstChild.nodeValue;
						if (app_icon)
							app_icon = launchyApps.substWinPaths(app_icon);
					}

					if (!app_name || !app_type || !app_exe) {
						launchyShared.trace("Invalid launchy.xml file. <label>, <type> and <command> are required.", 1);
						continue;
					}

					var ft = Launchy_FileIO.open(app_exe);
					if (ft && ft.exists() && ft.isFile()) {
						var dupe = 0;
						for (var k = 0; LaunchyContext.apps.length > k; k++) {
							if (LaunchyContext.apps[k].name == app_name) {
								if (LaunchyContext.apps[k].type != app_type) {
									launchyShared.trace("Reusing <" + app_name + "> detection");
									LaunchyContext.apps[LaunchyContext.apps.length] = launchyApps.createApp(app_name, LaunchyContext.apps[k].exe, app_type, app_args, app_icon);
									dupe = 1;
									break;
								} else {
									launchyShared.trace("<" + app_name + "> type <" + app_type + "> is already detected");
									dupe = 1;
									break;
								}
							}
						}
						if (!dupe) {
							launchyShared.trace("Found application: <" + app_name + "> type <" + app_type + "> at <" + app_exe + ">" + (app_args ? " with arguments <" + app_args + ">" : "") + (app_icon ? " with icon <" + app_icon + ">" : ""));
							LaunchyContext.apps[LaunchyContext.apps.length] = launchyApps.createApp(app_name, app_exe, app_type, app_args, app_icon);
						}
					} else {
						launchyShared.trace("Invalid launchy.xml file. Didn't find application: <" + app_name + "> type <" + app_type + "> at <" + app_exe + ">", 1);
					}
				}
			} else {
				launchyShared.trace("Invalid launchy.xml file. Inner object isn't <application>", 1);
			}
		} else {
			launchyShared.trace("Invalid launchy.xml file. Outer object isn't <configurations>", 1);
		}
	} else if (typeof(xmlfile) != "undefined") {
		launchyShared.trace("<" + xmlfile + "> does not exist");
	}
}

// create the object
const launchyApps = new LaunchyApps();
